import numpy as np
import matplotlib.pyplot as plt
import multiprocessing
from multiprocessing import Manager
from scipy.spatial import cKDTree
from pykdtree.kdtree import KDTree # pykdtree by Esben S. Nielsen 

class Particles():
    def __init__(self, pos,
                 mass = None,
                 hsml = None,
                 nb = 32,
                 verbose = False,
                 sort = False):
        """
        Particles class stores particle's information that will be used 
        later for rendering the image. 
        
        Input arguments are: particles position, mass and optionally smoothing lengths. 
        
        Positions of the particles are given by the array *pos* with shape [3,n], where
        n is the number of particles, and pos[0,:] = x, pos[1,:] = y and pos[2,:] = z. 
        
        If masses and smoothing lengths are not given, Particles class assumes the same mass = 1 for every particle, 
        and the corresponding smoothing length of each particle is computed as the distance to its "nb" closer neighbor. 
        By default nb=32. 

        Note that once you have created an instance of Particle, it is not necessary
        to instantiate it again in case you may want to change any property. 
        Particles class has its own method for setting and/or getting
        the properties of the particles already stored:

        The "setting" methods are:

        - :method:`set_pos(pos)`
        - :method:`set_mass(mass)`
        - :method:`set_hsml(hsml)`
        - :method:`set_nb(nb)`

        The "getting" methods are:
        
        - :method:`get_pos()`
        - :method:`get_mass()`
        - :method:`get_hsml()`
        - :method:`get_nb()`

        Finally, Particles class has its own plotting method:

        - :method:`plot('plane', axis=None, **kwargs)`

        in which 'plane' is one of the available projections of the input data:
        |'xy'|'xz'|'yz'|, and axis makes a reference to an existing axis. 
        If axis is None (default), the plot is made on the current active axis. 
        
        Please read the matplotlib.pyplot.plot documentation for the accepted
        The kwargs are :class:`~matplotlib.lines.Line2D` properties:
        """

        self._name = 'PARTICLES'

        self.__verbose = verbose

        npart = np.size(mass)

        if(hsml is None):
            hsml = self.__det_hsml(pos,nb)
        
        if(sort):
            ksort = np.argsort(hsml)
            self.__pos  = np.ascontiguousarray(pos[:,ksort])
            self.__mass = np.ascontiguousarray(mass[ksort])
            self.__hsml = np.ascontiguousarray(hsml[ksort])
        else:
            self.__pos  = np.ascontiguousarray(pos)
            self.__mass = np.ascontiguousarray(mass)
            self.__hsml = np.ascontiguousarray(hsml)


#Setting methods:
    def set_pos(self,pos):
        """
        Use this method to overwrite the stored array of particles.
        """
        self.__pos  = pos

    def set_mass(self,mass):
        """
        Use this method to overwrite the stored array of masses.
        """
        self.__mass  = mass
    
    def set_hsml(self,hsml):
        """
        Use this method to overwrite the stored array of smoothing lengths.
        """
        self.__hsml  = hsml

    def set_nb(self,nb):
        """
        Use this method to overwrite the defined number of neighbors
        to be used for computing the smoothing lengths.
        """
        self.__nb  = nb

#Getting methods
    def get_pos(self):
        """
        Use this method to get the stored array of particles.
        - Output: [3,n] numpy array with x = pos[0,:], y = pos[1,:], z = pos[2,:]
        with n the number of particles.
        """
        return self.__pos

    def get_mass(self):
        """
        Use this method to get the stored array of masses.
        """
        return self.__mass
    
    def get_hsml(self):
        """
        Use this method to get the stored array of smoothing lengths.
        """
        return self.__hsml

    def get_nb(self):
        """
        Use this method to get the number of neighbors that were used to 
        compute the smoothing lengths.
        """
        return self.__nb

    def plot(self,plane,axis=None,**kargs):
        """
        Use this method to plot the set of particles stored by the Particles class.
        In order to plot the distribution of Particles, a *plane* parameter must be given.
        "plane" is one of the available orthogonal projections of the particles:  
        |'xy'|'xz'|'yz'|. If there is multiple axes defined, the active one can be 
        selected using the axis parameter. If axis paremeter is None (default), the 
        distribution of particles is plotted in the active axis, which is returned by 
        the matplotlib.pyplot.gca() method.
        """
        if(axis == None):
            axis = plt.gca()
        if(plane == 'xy'):
            axis.plot(self.__pos[0,:], self.__pos[1,:], 'k.', **kargs)
        elif(plane == 'xz'):
            axis.plot(self.__pos[0,:], self.__pos[2,:], 'k.', **kargs)
        elif(plane == 'yz'):
            axis.plot(self.__pos[1,:], self.__pos[2,:], 'k.', **kargs)

    def __make_kdtree(self,pos):
        return KDTree(pos.T)
    
    def __nbsearch(self, pos, nb, tree):
        d, idx = tree.query(pos.T, k=nb)
        hsml = d[:,nb-1]
        return hsml
    
    def __det_hsml(self, pos, nb):
        tree = self.__make_kdtree(pos)
        hsml = self.__nbsearch(pos, nb, tree)
        return hsml

    def __make_kdtree_old(self,pos):
        return cKDTree(pos.T)

    def __nbsearch_old(self, pos, nb, tree, out_hsml, index):
        d, idx = tree.query(pos.T, k=nb)
        out_hsml.put( (index, d[:,nb-1]) )

    def __det_hsml_old(self, pos, nb):
        """
        Use this function to find the smoothing lengths of the particles.
        hsml = det_hsml(pos, nb)
        """
        manager = Manager()
        out_hsml  = manager.Queue()
        size  = multiprocessing.cpu_count()	

        if(self.__verbose): print 'Building a KDTree...'
        tree = self.__make_kdtree(pos)

        index  = np.arange(np.shape(pos)[1])
		#I split the job among the number of available processors
        pos   = np.array_split(pos, size, axis=1)	
        
        procs = []

        #We distribute the tasks among different processes
        if(self.__verbose): print 'Searching the ', nb, 'closer neighbors to each particle...'
        for rank in xrange(size):
            task = multiprocessing.Process(target=self.__nbsearch, 
                                           args=(pos[rank], nb, tree, 
                                                 out_hsml,rank))
            procs.append(task) 
            task.start()
            
            #Wait until all processes finish
        for p in procs:
            p.join()

            index = []
            hsml  = []
        for i in xrange(size):
            a, b = out_hsml.get()
            index.append(a)
            hsml.append(b)
    #	    if(a == 0): print b[0]			

            #I have to order the data before return it
        k = np.argsort(index)
        hsml1 = np.array([])
        for i in k:
            hsml1 = np.append(hsml1,hsml[i])
        if(self.__verbose): print 'Done...'
        return hsml1        



import numpy as np
import scipy.interpolate as interp

def get_snapshots_and_times(time, snaplist, timelist):
    current_snap   = np.interp(time, timelist, snaplist)
    lower_snapshot = np.floor(current_snap).astype(np.int32)
    upper_snapshot = np.floor(current_snap).astype(np.int32)+1
    if(upper_snapshot < len(timelist)):
        lower_time     = timelist[lower_snapshot]
        upper_time     = timelist[upper_snapshot]
    else:
        lower_time     = timelist[lower_snapshot-1]
        upper_time     = timelist[upper_snapshot-1]
    return lower_snapshot, upper_snapshot, lower_time, upper_time
                                

def same_pass(param, frames):
    while 'same' in param:
        idx = param.index('same')
        param[idx] = param[idx-1]

    while 'pass' in param:
        idx = param.index('pass')
        del param[idx]
        del frames[idx]

def get_camera_trayectory(targets,anchors):
    #Author: Alejandro Benitez-Llambay
    #I wrote this function to get the parameters of the camera 
    #that define its trayectory according to a small set of anchors. 
    #It is based on a first code written by Adrien Thob, who was 
    #inspired by the "Surge Target" plugin for Adobe After Effects.

    keys = ['sim_times', 'id_targets', 't', 'p', 'r', 'zoom','extent']
    f_interp = {}
    for key in keys:
        frames = [i for i in anchors['id_frames']]
        same_pass(anchors[key], frames)

        if(key == 'id_targets'):
            xc = []
            yc = []
            zc = []
            for i in anchors[key]:
                xc.append(targets[i][0])
                yc.append(targets[i][1])
                zc.append(targets[i][2])
            f_interp['x'] = interp.interp1d(frames, xc)
            f_interp['y'] = interp.interp1d(frames, yc)
            f_interp['z'] = interp.interp1d(frames, zc)
        else:
            f_interp[key] = interp.interp1d(frames, anchors[key])

    camera_params = []
    frames = np.arange(anchors['id_frames'][0], anchors['id_frames'][-1])
    camera_params = []
    keys = ['id_frames','sim_times','r','t','p','x','y','z', 'zoom','extent']
    for i in frames:
        params = {}
        for key in keys:
            if(key == 'id_frames'):
                params[key] = i
            elif(key == 'extent'):
                value = float(f_interp[key](i))
                params[key] = [-value, value, -value, value]
            else:
                params[key] = float(f_interp[key](i))
        camera_params.append(params)

    return camera_params

if __name__ == "__main__":
    import sphviewer as sph
    import matplotlib.pyplot as plt

    cm_1 = [0.5,1.5,0.5]
    cm_2 = [0.5,-5.5,0.5]

    targets = [cm_1, cm_2]

    anchors = {}
    anchors['sim_times'] = [0.0, 1.0, 'pass', 3.0, 'same','same','same']
    anchors['id_frames'] =  [0,180,750,840,930,1500,1680]
    anchors['r']         =  [10,2,'same',4,2,'same',10]
    anchors['id_targets']=  [0,1,'same','pass',0,'same',1]
    anchors['t']         = [0,'pass','pass',45,'pass','pass',0]
    anchors['p']         = [0,'pass','pass','pass','pass','pass',900]
    anchors['zoom']      = [1.,'same','same','same','same','same','same']
    anchors['extent']    = [10, 'pass','pass','pass','pass','pass',30]
    
    data = get_camera_trayectory(targets,anchors)

    n1 = 10000

    cube1 = np.random.rand(3,n1)
    cube1[1,:] -= 6 
    cube2 = np.random.rand(3,n1)
    cube2[1,:] += 1
    cubes = np.concatenate((cube1,cube2),axis=1)
    mass  = np.ones(n1+n1)

    P = sph.Particles(cubes, mass)
    S = sph.Scene(P)

    h = 0
    for i in data:
        i['xsize'] = 200
        i['ysize'] = 200
        i['roll'] = 0
        S = sph.Scene(P)
        S.update_camera(**i)
        print S.Camera.get_params()
        R = sph.Render(S)
        img = R.get_image()
        R.set_logscale()
        plt.imsave('test/image_'+str('%d.png'%h), img, vmin=0, vmax=6, cmap='cubehelix')
        h += 1
    
    
    



#from scipy import weave
#from scipy.weave import converters
import numpy as np
import matplotlib.pyplot as plt
import os
import colorsys

PROJECT_ROOT = os.path.abspath(os.path.dirname(__file__))

def import_code(filename):
    #naive function to import c code
    string = ''
    fi = open(filename, 'r').readlines()
    for i in fi:
        string += i
    return string


class Render():
    def __init__(self,Scene):
        """
        sphviewer.Render class renderizes a sphviewer.Scene class. It uses a kernel interpolation 
        method. It has setting and getting methods:
        
        getting methods:
        ----------------
        
        - get_image
        - get_min
        - get_max
        - get_extent
        - get_logscale
        
        setting methods:
        ----------------
        
        - set_logscale
                
        Other methods are:
        
        - save 
        - histogram

        """
        try:
            class_name = Scene._name
        except AttributeError:
            print "You must use a valid class..."
            return
        if(class_name != 'SCENE'):
            print "You must use a valid class..."
            return

        self.Scene = Scene
        x,y,t,kview = Scene.get_scene()
        xsize = Scene.Camera.get_params()['xsize']
        ysize = Scene.Camera.get_params()['ysize']

        self.__image = self.__make_render(x,y,t,kview,xsize,ysize)
        #lets define some flags
        self.__logscale_flag = False;


    def __make_render(self,x,y,t,kview,xsize,ysize):
        import extensions.render

        mass = self.Scene._Particles.get_mass()[kview]

        image = extensions.render.render(np.int32(x),np.int32(y),np.int32(t),np.float32(mass),np.int32(xsize),np.int32(ysize))
        return np.reshape(image,[ysize,xsize])


    def __make_render_old(self,x,y,t,kview,xsize,ysize):
        #Old function using weave for rendering the images. 

        n=int(len(x))
        mass = self.Scene._Particles.get_mass()[kview]

        image = np.zeros([ysize,xsize],dtype=(np.float32))

            # C code for making the images
        code = import_code(os.path.join(PROJECT_ROOT, '.','c_code.c'))
        shared     = (['x','y', 'xsize', 'ysize',  
                       't','n', 'mass','image'])
        # interpolation kernel

        extra_code = import_code(os.path.join(PROJECT_ROOT, '.','extra_code.c'))            
        weave.inline(code,shared,
                     support_code=extra_code,
                     type_converters=converters.blitz,
                     compiler='gcc',
                     headers=["<omp.h>"],
                     extra_compile_args=[' -O3 -fopenmp'],
                     extra_link_args=['-lgomp'])
        return image 

    def get_image(self):
        """
        - get_image(): This method returns the matrix of the image, whose dimension is [xsize,ysize], 
        where xsize and ysize are the number of pixels of the image defined as parameters of the camera.
        """
        return self.__image

    def get_max(self):
        """
        - get_max(): This method returns the maximum value found in the image.
        """
        return np.max(self.__image)

    def get_min(self):
        """
        - get_min(): This method returns the minimum value found in the image.
        """
        return np.min(self.__image)

    def get_extent(self):
        """
        - get_extent(): It calls to the method get_extent from the sphviewer.Scene class.
        """
        return self.Scene.get_extent()

    def set_logscale(self,t=True):
        """
        - set_logscale(): If M is the matrix of the image, it defines the image M as log10(M+1).
        """
        if(t == self.get_logscale()):
            return
        else:
            if(t):
                self.__image = np.log10(self.__image+1)
                self.__logscale_flag = True;
            else:
                self.__image = 10**self.__image-1.
                self.__logscale_flag = False;

    def get_logscale(self):
        """
        - get_logscale(): It returns True or False depending on the scale of the image.
        """
        return self.__logscale_flag

    def histogram(self,axis=None, **kargs):
        """
        - histogram(axis=None, **kargs): It computes and shows the histogram of the image. This is 
        usefull in order to choose a proper scale to the output, or for clipping some values. If 
        axis is None, it choose the current axis to plot the histogram.
        
        Keyword arguments:
        
        *bins*:
        Either an integer number of bins or a sequence giving the
        bins.  If *bins* is an integer, *bins* + 1 bin edges
        will be returned, consistent with :func:`numpy.histogram`
        for numpy version >= 1.3, and with the *new* = True argument
        in earlier versions.
        Unequally spaced bins are supported if *bins* is a sequence.
        
        *range*:
        The lower and upper range of the bins. Lower and upper outliers
        are ignored. If not provided, *range* is (x.min(), x.max()).
        Range has no effect if *bins* is a sequence.
        
        If *bins* is a sequence or *range* is specified, autoscaling
        is based on the specified bin range instead of the
        range of x.
        
        *normed*:
        If *True*, the first element of the return tuple will
        be the counts normalized to form a probability density, i.e.,
        ``n/(len(x)*dbin)``.  In a probability density, the integral of
        the histogram should be 1; you can verify that with a
        trapezoidal integration of the probability density function::
        
        pdf, bins, patches = ax.hist(...)
        print np.sum(pdf * np.diff(bins))
        
        .. note::
        
        Until numpy release 1.5, the underlying numpy
        histogram function was incorrect with *normed*=*True*
        if bin sizes were unequal.  MPL inherited that
        error.  It is now corrected within MPL when using
        earlier numpy versions
        
        *weights*:
        An array of weights, of the same shape as *x*.  Each value in
        *x* only contributes its associated weight towards the bin
        count (instead of 1).  If *normed* is True, the weights are
        normalized, so that the integral of the density over the range
        remains 1.
        
        *cumulative*:
        If *True*, then a histogram is computed where each bin
        gives the counts in that bin plus all bins for smaller values.
        The last bin gives the total number of datapoints.  If *normed*
        is also *True* then the histogram is normalized such that the
        last bin equals 1. If *cumulative* evaluates to less than 0
        (e.g. -1), the direction of accumulation is reversed.  In this
        case, if *normed* is also *True*, then the histogram is normalized
        such that the first bin equals 1.
        
        *histtype*: [ 'bar' | 'barstacked' | 'step' | 'stepfilled' ]
        The type of histogram to draw.
        
        - 'bar' is a traditional bar-type histogram.  If multiple data
        are given the bars are aranged side by side.
        
        - 'barstacked' is a bar-type histogram where multiple
        data are stacked on top of each other.
        
        - 'step' generates a lineplot that is by default
        unfilled.
        
        - 'stepfilled' generates a lineplot that is by default
        filled.
        
        *align*: ['left' | 'mid' | 'right' ]
        Controls how the histogram is plotted.
        
        - 'left': bars are centered on the left bin edges.
        
        - 'mid': bars are centered between the bin edges.
        
        - 'right': bars are centered on the right bin edges.
        
        *orientation*: [ 'horizontal' | 'vertical' ]
        If 'horizontal', :func:`~matplotlib.pyplot.barh` will be
        used for bar-type histograms and the *bottom* kwarg will be
        the left edges.
        
        *rwidth*:
        The relative width of the bars as a fraction of the bin
        width.  If *None*, automatically compute the width. Ignored
        if *histtype* = 'step' or 'stepfilled'.
        
        *log*:
        If *True*, the histogram axis will be set to a log scale.
        If *log* is *True* and *x* is a 1D array, empty bins will
        be filtered out and only the non-empty (*n*, *bins*,
        *patches*) will be returned.
        
        *color*:
        Color spec or sequence of color specs, one per
        dataset.  Default (*None*) uses the standard line
        color sequence.
        
        *label*:
        String, or sequence of strings to match multiple
        datasets.  Bar charts yield multiple patches per
        dataset, but only the first gets the label, so
        that the legend command will work as expected::
        
        ax.hist(10+2*np.random.randn(1000), label='men')
        ax.hist(12+3*np.random.randn(1000), label='women', alpha=0.5)
        ax.legend()
        
        
        kwargs are used to update the properties of the
        :class:`~matplotlib.patches.Patch` instances returned by *hist*:
        
        agg_filter: unknown
        alpha: float or None         
        animated: [True | False]         
        antialiased or aa: [True | False]  or None for default         
        axes: an :class:`~matplotlib.axes.Axes` instance         
        clip_box: a :class:`matplotlib.transforms.Bbox` instance         
        clip_on: [True | False]         
        clip_path: [ (:class:`~matplotlib.path.Path`,         :class:`~matplotlib.transforms.Transform`) |         :class:`~matplotlib.patches.Patch` | None ]         
        color: matplotlib color spec
        contains: a callable function         
        edgecolor or ec: mpl color spec, or None for default, or 'none' for no color         
        facecolor or fc: mpl color spec, or None for default, or 'none' for no color         
        figure: a :class:`matplotlib.figure.Figure` instance         
        fill: [True | False]         
        gid: an id string         
        hatch: [ '/' | '\\' | '|' | '-' | '+' | 'x' | 'o' | 'O' | '.' | '*' ]         
        label: any string         
        linestyle or ls: ['solid' | 'dashed' | 'dashdot' | 'dotted']         
        linewidth or lw: float or None for default         
        lod: [True | False]         
        path_effects: unknown
        picker: [None|float|boolean|callable]         
        rasterized: [True | False | None]         
        snap: unknown
        transform: :class:`~matplotlib.transforms.Transform` instance         
        url: a url string         
        visible: [True | False]         
        zorder: any number 
        """
        if(axis == None):
            axis = plt.gca()
        axis.hist(self.__image.ravel(), **kargs)
        
    def save(self,outputfile,**kargs):
        """
        - save allows to save the image in any of the 
        common image formats. It uses the pyplot.save 
        method. 
        outputfile is a string containing a path to a filename, 
        of a Python file-like object. If *format* is *None* and
        *fname* is a string, the output format is deduced from
        the extension of the filename.

        Keyword arguments:
        *vmin*/*vmax*: [ None | scalar ]
        *vmin* and *vmax* set the color scaling for the image by fixing the
        values that map to the colormap color limits. If either *vmin* or *vmax*
        is None, that limit is determined from the *arr* min/max value.
        *cmap*:
        cmap is a colors.Colormap instance, eg cm.jet.
        If None, default to the rc image.cmap value.
        *format*:
        One of the file extensions supported by the active
        backend.  Most backends support png, pdf, ps, eps and svg.
        *origin*
        [ 'upper' | 'lower' ] Indicates where the [0,0] index of
        the array is in the upper left or lower left corner of
        the axes. Defaults to the rc image.origin value.
        *dpi*
        The DPI to store in the metadata of the file.  This does not affect the
        resolution of the output image.
        """
        plt.imsave(outputfile, self.__image, **kargs)

        




import numpy as np
import matplotlib.pyplot as plt

#Tools for blending images. There are plenty of possibilities, and
#depending on the images, the final result can vary significantly.

class Blend():
    def __init__(self, image1, image2):
        self.image1 = image1
        self.image2 = image2


    def Screen(self):
        output = np.zeros(np.shape(self.image1))

        if np.shape(self.image1)[2] == 4:  # test for RGBA input image
            output[:,:,3] = 1  # set alpha values to unity

        for i in xrange(3):
            output[:,:,i] = (1.0-(1.0-self.image1[:,:,i])*
                                 (1.0-self.image2[:,:,i]))
        return output

    def Overlay(self):
        output = np.zeros(np.shape(self.image1))

        if np.shape(self.image1)[2] == 4:  # test for RGBA input image
            output[:,:,3] = 1  # set alpha values to unity

        for i in xrange(3):
            output[:,:,i] = (self.image1[:,:,i]*
                            (self.image1[:,:,i]+2*self.image2[:,:,i]*
                            (1.0-self.image1[:,:,i])) )

        return output



if __name__ == "__main__":
    import numpy as np
    import matplotlib.pyplot as plt
    from sphviewer.tools import QuickView
    from matplotlib import cm

    def get_normalized_image(image):
        image = (image-np.min(image))/(np.max(image)-np.min(image))
        return image

    pos1 = np.zeros([5000,3])
    pos2 = np.zeros([10000,3])

    pos1[:,0:2] = -1+2*np.random.rand(5000,2)
    pos2[:,0:2] = -1+2*np.random.rand(10000,2)


    r2 = np.sqrt(pos2[:,0]**2+pos2[:,1]**2)
    k2, = np.where(r2 < 0.5)
    pos2 = pos2[k2,:]

    qv1 = QuickView(pos1.T, np.ones(len(pos1)),
                    r='infinity', logscale=False, plot=False,
                    extent=[-1,1,-1,1], x=0, y=0, z=0)
    qv2 = QuickView(pos2.T, np.ones(len(pos2)),
                    r='infinity', logscale=False, plot=False,
                    extent=[-1,1,-1,1], x=0, y=0, z=0)


    image1 = cm.gist_heat(get_normalized_image(qv1.get_image()))
    image2 = cm.gist_stern(get_normalized_image(qv2.get_image()))


    fig = plt.figure(1, figsize=(10,5))
    ax1 = fig.add_subplot(121)
    ax2 = fig.add_subplot(122)

    blend = Blend(image1, 1.0*image2)
    screen  = blend.Screen()
    overlay = blend.Overlay()

    ax1.imshow(screen, origin='lower', extent=qv1.get_extent())
    ax1.set_title('Screen')
    ax2.imshow(overlay, origin='lower', extent=qv1.get_extent())
    ax2.set_title('Overlay')
    plt.show()




import numpy as np
import matplotlib.pylab as plt

class Camera():
    def __init__(self, x = None, y = None, z = None, r = None, 
                 t = None, p = None, zoom  = None, roll = None,
                 xsize = None, ysize = None, extent = None):
        self._name = 'CAMERA'        
        self.__params = {'x':x,'y':y,'z':z,'r':r,
                         't':t,'p':p,'zoom':zoom, 'roll':roll,
                         'xsize':xsize, 'ysize':ysize, 'extent':extent}

    def get_params(self):
        return self.__params
    
    def set_params(self, **kargs):
            for key in kargs:
                self.__params[key] = kargs[key]

    def _get_camera(self,plane, **kargs):
        x0 = -self.__params['r']*(np.sin(self.__params['p']*np.pi/180.)*
                                 np.cos(self.__params['t']*np.pi/180.))
        z0 = self.__params['r']*(np.cos(self.__params['p']*np.pi/180.)*
                                np.cos(self.__params['t']*np.pi/180.))
        y0 = self.__params['r']*np.sin(self.__params['t']*np.pi/180.)
        
        xcam = self.__params['x']+x0
        ycam = self.__params['y']+y0
        zcam = self.__params['z']+z0
        
        
        if(plane == 'xy'):
            camera = plt.Line2D([xcam],[ycam], c='m', marker='o', markersize=15, **kargs)
            arrow =  plt.Line2D([xcam,self.__params['x']],[ycam,self.__params['y']],lw=5)
        elif(plane == 'xz'):
            camera = plt.Line2D([xcam],[zcam], c='m', marker='o', markersize=15, **kargs)
            arrow =  plt.Line2D([xcam,self.__params['x']],[zcam,self.__params['z']],lw=5)
        elif(plane == 'yz'):
            camera = plt.Line2D([ycam],[zcam], c='m', marker='o', markersize=15, **kargs)
            arrow =  plt.Line2D([ycam,self.__params['y']],[zcam,self.__params['z']],lw=5)
        else:
            print 'Incorrect plane:', plane
            print "Possibles planes are: 'xy';'xz';'yz'"
            return
        return camera, arrow

    def plot(self,plane, axis=None, **kargs):
        if(axis == None):
            axis = plt.gca()
        camera, arrow = self._get_camera(plane)
        axis.add_line(camera, **kargs)
        axis.add_line(arrow, **kargs)

    def set_autocamera(self,Particles, mode='minmax'):
        try:
            particles_name = Particles._name
        except AttributeError:
            print "You must use a valid class..."
            return
        if (particles_name != 'PARTICLES'):
            print "You must use a valid Particles class..."
            return
        xmax,ymax,zmax = (np.max(Particles.get_pos()[0,:]),
                          np.max(Particles.get_pos()[1,:]),
                          np.max(Particles.get_pos()[2,:]))
        xmin,ymin,zmin = (np.min(Particles.get_pos()[0,:]),
                          np.min(Particles.get_pos()[1,:]),
                          np.min(Particles.get_pos()[2,:]))

        if(mode == 'minmax'):
            xmean = (xmax+xmin)/2.
            ymean = (ymax+ymin)/2.
            zmean = (zmax+zmin)/2.
            
        if(mode == 'density'):
            k = np.argmin(Particles.get_hsml()[:])
            xmean = Particles.get_pos()[0,k]
            ymean = Particles.get_pos()[1,k]
            zmean = Particles.get_pos()[2,k]

        if(mode == 'median'):
            xmean = np.median(Particles.get_pos()[0,:])
            ymean = np.median(Particles.get_pos()[1,:])
            zmean = np.median(Particles.get_pos()[2,:])

        if(mode == 'mean'):
            xmean = np.mean(Particles.get_pos()[0,:])
            ymean = np.mean(Particles.get_pos()[1,:])
            zmean = np.mean(Particles.get_pos()[2,:])

        r = np.sqrt((xmax-xmin)**2+(ymax-ymin)**2+(zmax-zmin)**2)

        self.__params = {'x':xmean,'y':ymean,'z':zmean,'r':r,
                         't':0,'p':0,'zoom':1, 'roll':0,
                         'xsize':500, 'ysize':500, 'extent':None}



#//////////////////////////////////////////////////////
#This tools is part of py-sphviewer. 
#Author: Alejandro Benitez-Llambay
#//////////////////////////////////////////////////////
from makehsv import makehsv
import numpy as np

def image_from_hsv(h = 0, v = 0, 
                   img_hmin = None, img_hmax=None,
                   img_vmin = None, img_vmax=None,
                   hmin = None, hmax=None):

    if(img_hmin == None): img_hmin = np.min(h)
    if(img_hmax == None): img_hmax = np.max(h)
    if(img_vmin == None): img_vmin = np.min(v)
    if(img_vmax == None): img_vmax = np.max(v)
    if(hmin  == None): hmin = 0.0
    if(hmax  == None): hmax = 1.0

    ysize = np.shape(v)[1]
    xsize = np.shape(v)[0]

    image = np.zeros([ysize,xsize,3], dtype=np.float32)

    r, g, b= makehsv(h, v, img_hmin, img_hmax,
                     img_vmin, img_vmax,
                     hmin, hmax)

    image[:,:,0] = np.reshape(r, [ysize,xsize])
    image[:,:,1] = np.reshape(g, [ysize,xsize])
    image[:,:,2] = np.reshape(b, [ysize,xsize])

    return image



#This tool is part of sphviewer. It is intended to be used as a simple
#way to get quick images of the simulations.
#Author: Alejandro Benitez-Llambay

import sphviewer as sph
import matplotlib.pyplot as plt
import numpy as np

class QuickView():
    def    __init__(self, pos, mass=None, hsml=None, nb=None,
                    logscale=True, plot=True, min_hsml=None, max_hsml=None, **kwargs):

        if(mass is None):
            mass = np.ones(len(pos[0,:]))
                    
        if(nb == None):
            self._P = sph.Particles(pos, mass, hsml)
        else:
            self._P = sph.Particles(pos, mass, hsml, nb)

        if( (min_hsml is not None) or (max_hsml is not None) ):
            hsml = self.get_hsml()
            if(min_hsml is not None):
                min_hsml = min_hsml
            else:
                min_hsml = np.min(hsml)
            if(max_hsml is not None):
                max_hsml = max_hsml
            else:
                max_hsml = np.max(hsml)

            hsml = np.clip(hsml, min_hsml, max_hsml)
            print 'Limiting smoothing lenght to the range [%.3f,%.3f]'%(min_hsml, max_hsml)
            self._P.set_hsml(hsml)
            
        self._S = sph.Scene(self._P)
        self._S.update_camera(**kwargs)

        self._R = sph.Render(self._S)
        if(logscale):
            self._R.set_logscale()

        self._img = self._R.get_image()
        self._extent = self._R.get_extent()

        if(plot):
            self.imshow(aspect='auto')
            return
        else:
            return
            
    def imshow(self, **kwargs):
        ax = plt.gca()
        ax.imshow(self._img, extent=self._extent, origin='lower', **kwargs)
        plt.show()

    def get_image(self):
        return self._img

    def get_extent(self):
        return self._extent

    def imsave(self, filename, **kwargs):
        try:
            plt.imsave(filename, self._img, **kwargs)
            print 'Image saved in '+ filename
        except:
            print 'Error while saving image'
        return

    def get_hsml(self):
        return self._P.get_hsml()
        
        
if __name__ == '__main__':
    import h5py
    halo = h5py.File('../../examples/dm_halo.h5py', 'r')
    pos  = halo['Coordinates'].value

    qv = QuickView(pos.T, r='infinity', nb=8)
    



#from distutils.core import setup, Extension
try:
    from setuptools import setup, Extension
except ImportError:
    from distutils.core import setup, Extension

import numpy as np

module_scene = Extension('sphviewer/extensions/scene', sources = ['sphviewer/extensions/scenemodule.c'],
                         extra_compile_args=['-fopenmp','-w'],
                         extra_link_args=['-lgomp'])

module_render = Extension('sphviewer/extensions/render', sources = ['sphviewer/extensions/rendermodule.c'],
                          extra_compile_args=['-fopenmp','-w'],
                          extra_link_args=['-lgomp'])

module_makehsv = Extension('sphviewer/tools/makehsv', sources = ['sphviewer/tools/makehsvmodule.c'],
                           extra_compile_args=['-fopenmp','-w'],
                           extra_link_args=['-lgomp'])


exec(open('sphviewer/version.py').read())

setup(name='py-sphviewer',
      version=__version__,
      description='Py-SPHViewer is a framework for rendering particles using the smoothed particle hydrodynamics scheme.',
      author='Alejandro Benitez Llambay',
      author_email='alejandrobll@oac.uncor.edu',
      url='https://github.com/alejandrobll/py-sphviewer',
      packages=['sphviewer','sphviewer.extensions', 'sphviewer.tools'],
      include_dirs = [np.get_include()],
      requires = ['pykdtree'],
      install_requires = ['pykdtree'],
      package_data={'sphviewer': ['*.c','*.txt']},
      ext_modules = [module_scene, module_render, module_makehsv],
      license='GNU GPL v3',
      classifiers=[
        'Programming Language :: Python :: 2.7',
        'Topic :: Multimedia :: Graphics',
        'Topic :: Multimedia :: Graphics :: 3D Rendering',
        'Topic :: Multimedia :: Graphics :: Viewers',
        'Topic :: Scientific/Engineering :: Visualization',
        'Topic :: Utilities'
            ],
      keywords="smoothed particle hydrodynamics render particles nbody galaxy formation dark matter sph cosmology movies",      
     )



import numpy as np
import matplotlib.pyplot as plt

def rotate(angle, axis, pos):
    angle *= np.pi/180.0
    if axis == 'x':
        R = np.array([[1,0,0],
                      [0,np.cos(angle),np.sin(angle)],
                      [0,-np.sin(angle),np.cos(angle)]])
    elif axis == 'y':
        R = np.array([[np.cos(angle),0,-np.sin(angle)],
                      [0,1,0],
                      [np.sin(angle),0,np.cos(angle)]])
    elif axis == 'z':
        R = np.array([[np.cos(angle),np.sin(angle),0],
                      [-np.sin(angle),np.cos(angle),0],
                      [0,0,1]])
    return np.dot(R,pos)

class Scene():
    def __init__(self, Particles, Camera=None):
        """
        Scene class takes a sphviewer.Particles class and computes the 
        coordinates of the particles as seen from a Camera. It is to say, 
        for a given particle, whose coordinates are x,y and z, Scene 
        computes a new set of coordinates x' and y', which are the aparent 
        coordinates of the particles as seen from a camera. 
        By default, Camera=None means that Scene have to use an autocamera. It 
        will try to define a Camera whose parameters have some sence according
        to your data set. In case you want to prevent Scene from using an autocamera, 
        you should provide a valid Camera Class.
        As Particles class, Scene has its own getting and setting methods.

        setting methods:
        ----------------

        - set_autocamera
        - update_camera

        getting methods:
        ----------------
        - get_scene
        - get_extent

        other methods are: 

        - plot

        """
        try:
            particles_name = Particles._name
        except AttributeError:
            print "You must use a valid Particles class..."
            return
        if(particles_name != 'PARTICLES'):
            print "You must use a valid Particles class..."
            return

        self._name = 'SCENE'
        self._Particles = Particles

        #I use the autocamera by default
        if(Camera==None):
            from Camera import Camera
            self.Camera = Camera()
            self.Camera.set_autocamera(Particles)
            self._camera_params = self.Camera.get_params()

        else:
            try: 
                camera_name = Camera._name
                if(camera_name != 'CAMERA'):
                    print "You must use a valid Camera class..."
                    print "I will try to set an autocamera..."
                    from Camera import Camera
                    self.Camera = Camera()
                    self.Camera.set_autocamera(Particles)
                    self._camera_params = self.Camera.get_params()
                else:
                    self.Camera = Camera
                    self._camera_params = self.Camera.get_params()
            except AttributeError:
                print "You must use a valid Camera class..."
                print "I will try to set an autocamera..."
                from Camera import Camera
                self.Camera = Camera()
                self.Camera.set_autocamera(Particles)
                self._camera_params = self.Camera.get_params()

        self.__x, self.__y, self.__hsml, self.__kview = self.__compute_scene()

    def set_autocamera(self,mode='density'):
        """
        - set_autocamera(mode='density'): By default, Scene defines its 
        own Camera. However, there is no a general way for doing so. Scene 
        uses a density criterion for getting the point of view. If this is 
        not a good option for your problem, you can choose among:
        |'minmax'|'density'|'median'|'mean'|. If None of the previous methods
        work well, you may define the camera params by yourself.
        """
        self.Camera.set_autocamera(self._Particles,mode=mode)
        self._camera_params = self.Camera.get_params()
        self.__x, self.__y, self.__hsml, self.__kview = self.__compute_scene()

    def get_scene(self):
        """
        - get_scene(): It return the x and y position, the smoothing length 
        of the particles and the index of the particles that are active in 
        the scene. In principle this is an internal function and you don't 
        need this data. 
        """
        return self.__x, self.__y, self.__hsml, self.__kview

    def get_extent(self):
        """
        - get_extent(): It returns the extent array needed for converting 
        the image coordinates (given in pixels units) into physical coordinates. 
        It is an array like the following one: [xmin,xmax,ymin,ymax]; it is to say, 
        an array that contains the extreme values of the scene. 
        """
        return self.__extent

    def update_camera(self,**kargs):
        """
        - update_camera(**kwarg): By using this method you can define all 
        the new paramenters of the camera. Read the available **kwarg in 
        the sphviewer.Camera documentation. 
        """
        self.Camera.set_params(**kargs)
        self.__x, self.__y, self.__hsml, self.__kview = self.__compute_scene()

    def __compute_scene(self):
        import extensions.scene as scene

        pos = self._Particles.get_pos().astype(np.float32)
        hsml = self._Particles.get_hsml().astype(np.float32)

        #I recast the variables just in case
        xcam  = np.float32(self._camera_params['x'])
        ycam  = np.float32(self._camera_params['y'])
        zcam  = np.float32(self._camera_params['z'])
        theta = np.float32(self._camera_params['t'])
        phi   = np.float32(self._camera_params['p'])
        roll  = np.float32(self._camera_params['roll'])
        zoom  = np.float32(self._camera_params['zoom'])
        xsize = np.int32(self._camera_params['xsize'])
        ysize = np.int32(self._camera_params['ysize'])
                
        if(self._camera_params['r'] == 'infinity'):
            projection = 0
            rcam = np.float32(0)
            if(self._camera_params['extent'] == None):
                xmin = np.min(pos[0,:]).astype(np.float32)
                xmax = np.max(pos[0,:]).astype(np.float32)
                ymin = np.min(pos[1,:]).astype(np.float32)
                ymax = np.max(pos[1,:]).astype(np.float32)
                self.__extent = np.array([xmin-xcam,
                                          xmax-xcam,
                                          ymin-ycam,
                                          ymax-ycam],dtype=np.float32)
            else:
                self.__extent = np.float32(self._camera_params['extent'])
        else:
            projection = 1
            rcam = np.float32(self._camera_params['r'])
            self.__extent = np.array([0,0,0,0],dtype=np.float32)

        xx,yy,hh,kk = scene.scene(pos[0,:],pos[1,:],pos[2,:],hsml,
                                  xcam, ycam, zcam, rcam, theta, phi, roll,
                                  zoom, self.__extent, xsize, ysize, projection)

        return xx,yy,hh,kk



    def __compute_scene_old(self):
        
        pos = (1.0*self._Particles.get_pos()).astype(np.float32)
        
        pos[0,:] -= np.array([self._camera_params['x']])
        pos[1,:] -= np.array([self._camera_params['y']])
        pos[2,:] -= np.array([self._camera_params['z']])

        if self._camera_params['t'] != 0:
            pos = rotate(self._camera_params['t'],'x',pos)
        if self._camera_params['p'] != 0:
            pos = rotate(self._camera_params['p'],'y',pos)
        if self._camera_params['roll'] != 0:
            pos = rotate(self._camera_params['roll'],'z',pos)

        if(self._camera_params['r'] == 'infinity'):
            try:
                extent = self._camera_params['extent']
                if(extent == None):
                    xmax = np.max(pos[0,:])
                    xmin = np.min(pos[0,:])
                    ymax = np.max(pos[1,:])
                    ymin = np.min(pos[1,:])

                    lmax = max(xmax,ymax)
                    lmin = min(ymax,ymin)

                    xmin = ymin = lmin
                    xmax = ymax = ymax
                else:
                    xmin = float(extent[0])
                    xmax = float(extent[1])
                    ymin = float(extent[2])
                    ymax = float(extent[3])


                self.__extent = np.array([xmin+self._camera_params['x'],
                                          xmax+self._camera_params['x'],
                                          ymin+self._camera_params['y'],
                                          ymax+self._camera_params['y']])

                lbin = 2*xmax/self._camera_params['xsize']

                kview, = np.where( (pos[0,:] > xmin) & (pos[0,:] < xmax) &
                                   (pos[1,:] > ymin) & (pos[1,:] < ymax) )

                pos   = pos[:,kview]
                hsml  = self._Particles.get_hsml()[kview]/lbin

                pos[0,:] = (pos[0,:]-xmin)/(xmax-xmin)*self._camera_params['xsize']
                pos[1,:] = (pos[1,:]-ymin)/(ymax-ymin)*self._camera_params['ysize']
            
            except AttributeError:
                print "There was an error with the extent of the Camera"
                return
                    
            return pos[0,:], pos[1,:], hsml, kview

        else:
            pos[2,:] -= (-1.0*self._camera_params['r'])
        
            FOV  = 2.*np.abs(np.arctan(1./self._camera_params['zoom']))
        
            xmax = self._camera_params['zoom']*np.tan(FOV/2.)
            xmin = -xmax
            ymax = 0.5*(xmax-xmin)*self._camera_params['ysize']/self._camera_params['xsize']
            ymin = -ymax
            xfovmax =  FOV/2.*180./np.pi
            xfovmin =  -FOV/2.*180./np.pi
            # in order to have symmetric y limits
            yfovmax = 0.5*((xfovmax-xfovmin)*
                           self._camera_params['ysize']/self._camera_params['xsize'])
            yfovmin = -yfovmax
            self.__extent = np.array([xfovmin,xfovmax,yfovmin,yfovmax])
            lbin = 2*xmax/self._camera_params['xsize']
            
            kview, = np.where((pos[2,:] > 0.) & 
                              (np.abs(pos[1,:])<=(np.abs(pos[2,:])* 
                                                  np.tan(FOV/2.))) &
                              (np.abs(pos[1,:]) <= (np.abs(pos[2,:])*
                                                    np.tan(FOV/2.))))
            pos   = pos[:,kview]
            hsml  = self._Particles.get_hsml()[kview]
        
            pos[0,:] = ((pos[0,:]*self._camera_params['zoom']/ 
                         pos[2,:]-xmin)/(xmax-xmin)*
                        (self._camera_params['xsize']-1.))
            pos[1,:] = ((pos[1,:]*self._camera_params['zoom']/ 
                         pos[2,:]-ymin)/(ymax-ymin)*
                        (self._camera_params['ysize']-1.))
            hsml = (hsml*self._camera_params['zoom']/pos[2,:]/lbin)
            
            return pos[0,:], pos[1,:], hsml, kview

    
    def plot(self,axis=None,**kargs):
        """
        - plot(axis=None, **kwarg): Finally, sphviewer.Scene class has its own plotting method. 
        It shows the scene as seen by the camera. It is to say, it plots the particles according
        to their aparent coordinates; axis makes a reference to an existing axis. In case axis is None,
        the plot is made on the current axis.

        The kwargs are :class:`~matplotlib.lines.Line2D` properties:

        agg_filter: unknown
        alpha: float (0.0 transparent through 1.0 opaque)         
        animated: [True | False]         
        antialiased or aa: [True | False]         
        axes: an :class:`~matplotlib.axes.Axes` instance         
        clip_box: a :class:`matplotlib.transforms.Bbox` instance         
        clip_on: [True | False]         
        clip_path: [ (:class:`~matplotlib.path.Path`,         :class:`~matplotlib.transforms.Transform`) |         :class:`~matplotlib.patches.Patch` | None ]         
        color or c: any matplotlib color         
        contains: a callable function         
        dash_capstyle: ['butt' | 'round' | 'projecting']         
        dash_joinstyle: ['miter' | 'round' | 'bevel']         
        dashes: sequence of on/off ink in points         
        data: 2D array (rows are x, y) or two 1D arrays         
        drawstyle: [ 'default' | 'steps' | 'steps-pre' | 'steps-mid' | 'steps-post' ]         
        figure: a :class:`matplotlib.figure.Figure` instance         
        fillstyle: ['full' | 'left' | 'right' | 'bottom' | 'top']         
        gid: an id string         
        label: any string         
        linestyle or ls: [ ``'-'`` | ``'--'`` | ``'-.'`` | ``':'`` | ``'None'`` | ``' '`` | ``''`` ]         and any drawstyle in combination with a linestyle, e.g. ``'steps--'``.         
        linewidth or lw: float value in points         
        lod: [True | False]         
        marker: [ ``7`` | ``4`` | ``5`` | ``6`` | ``'o'`` | ``'D'`` | ``'h'`` | ``'H'`` | ``'_'`` | ``''`` | ``'None'`` | ``' '`` | ``None`` | ``'8'`` | ``'p'`` | ``','`` | ``'+'`` | ``'.'`` | ``'s'`` | ``'*'`` | ``'d'`` | ``3`` | ``0`` | ``1`` | ``2`` | ``'1'`` | ``'3'`` | ``'4'`` | ``'2'`` | ``'v'`` | ``'<'`` | ``'>'`` | ``'^'`` | ``'|'`` | ``'x'`` | ``'$...$'`` | *tuple* | *Nx2 array* ]
        markeredgecolor or mec: any matplotlib color         
        markeredgewidth or mew: float value in points         
        markerfacecolor or mfc: any matplotlib color         
        markerfacecoloralt or mfcalt: any matplotlib color         
        markersize or ms: float         
        markevery: None | integer | (startind, stride)
        picker: float distance in points or callable pick function         ``fn(artist, event)``         
        pickradius: float distance in points         
        rasterized: [True | False | None]         
        snap: unknown
        solid_capstyle: ['butt' | 'round' |  'projecting']         
        solid_joinstyle: ['miter' | 'round' | 'bevel']         
        transform: a :class:`matplotlib.transforms.Transform` instance         
        url: a url string         
        visible: [True | False]         
        xdata: 1D array         
        ydata: 1D array         
        zorder: any number         
        
        kwargs *scalex* and *scaley*, if defined, are passed on to
        :meth:`~matplotlib.axes.Axes.autoscale_view` to determine
        whether the *x* and *y* axes are autoscaled; the default is
        *True*.
        
        Additional kwargs: hold = [True|False] overrides default hold state
        """
        if(axis == None):
            axis = plt.gca()
        axis.plot(self.__x, self.__y, 'k.', **kargs)
        



"""
Py-SPHViewer is an object-oriented rendering library. It was developed mainly
for rendering cosmological Smoothed Particles Hydrodynamical (SPH) simulations of galaxy formation, but in its current version, it can renderize any set of particles. 

Author: Alejandro Benitez-Llambay
E-mail: If you have any question, or you want to report bugs, issues, etc., please contact me at alejandrobll@oac.uncor.edu.
Acknowledgment: Many thanks to Pablo Benitez-Llambay. He has improved the original idea a lot, and without his help, Py-SPHViewer would not be what it is. 
"""

from Particles import Particles
from Camera import Camera
from Scene import Scene
from Render import Render
import numpy as np
import matplotlib.pyplot as plt
from version import __version__

if __name__ == '__main__':
    import time
    n1 = 10000 #number of particles to make the disk
    n2 = n1/3  # number of particles to make the background
    r  = np.random.rand(n1)
    phi = np.pi*np.random.rand(n1)
    pos = np.zeros([3,n1], dtype=np.float32)
    pos[0,:] = r*np.cos(phi)
    pos[1,:] = r*np.sin(phi)
    pos[2,:] = 0.1*np.random.rand(n1)
    
    background = -2+4*np.random.rand(3,n2)
    pos = np.concatenate((pos,background),axis=1)
    
    mass = np.ones(n1+n2)

    Particles1 = Particles(pos,mass,sort=True)
    Scene1 = Scene(Particles1)
    Render1 = Render(Scene1)
    for i in xrange(100):
        start = time.time()
        Scene1.update_camera(r=2.00, t=90, roll=30, xsize=1920, ysize=1080)
        Render1 = Render(Scene1)
        Render1.set_logscale()
        img = Render1.get_image()
        extent = Render1.get_extent()
        stop = time.time()
        print 'Time = ', stop-start
        fig = plt.figure(1,figsize=(5,5))
        ax1 = fig.add_subplot(111)
        ax1.imshow(img, extent=extent, origin='lower', cmap='hot')
        ax1.set_xlabel('X', size=15)
        ax1.set_ylabel('Y', size=15)
        
        plt.show()
